package digi.recipeManager;

import java.util.*;
import java.util.Map.Entry;

import org.bukkit.*;
import org.bukkit.block.*;
import org.bukkit.inventory.ItemStack;

import digi.recipeManager.data.Smelt;

class FurnaceTask implements Runnable
{
	private static int taskId = 0;
	
	protected static void start()
	{
		if(taskId > 0)
			Main.bs.cancelTask(taskId);
		
		if(Recipes.smeltCustomRecipes > 0)
			taskId = Main.bs.scheduleSyncRepeatingTask(Main.plugin, new FurnaceTask(), 0, Main.cfgFurnaceTicks);
	}
	
	@Override
	public void run()
	{
		final Iterator<Entry<Location, Double>> furnaces = Main.furnacesSmelting.entrySet().iterator();
		Entry<Location, Double> entry;
		Block block;
		Material blockType;
		BlockState blockState;
		Furnace furnace;
		ItemStack[] items;
		Smelt recipe;
		int furnaceTime;
		double time;
		
		while(furnaces.hasNext())
		{
			entry = furnaces.next();
			block = entry.getKey().getBlock();
			
			if(block == null)
			{
				furnaces.remove();
				continue;
			}
			
			blockType = block.getType();
			
			if(blockType != Material.FURNACE && blockType != Material.BURNING_FURNACE)
			{
				furnaces.remove();
				continue;
			}
			
			blockState = block.getState();
			
			if(!(blockState instanceof Furnace))
			{
				furnaces.remove();
				continue;
			}
			
			furnace = (Furnace)blockState;
			
			if(furnace.getBurnTime() <= 0)
			{
				furnaces.remove();
				continue;
			}
			
			items = furnace.getInventory().getContents();
			
			// if there's nothing to smelt, skip furnace
			if(items[0] == null || items[0].getType() == Material.AIR)
				continue;
			
			recipe = Recipes.getSmeltRecipe(items[0]);
			
			// No custom recipe for item or it has default time
			if(recipe == null || recipe.minTime <= -1.0)
				continue;
			
			// If the result already in the furnace is something else than the recipe OR result is already max stacks, skip furnace
			if(items[2] != null && (!recipe.result.compareItemStack(items[2]) || items[2].getAmount() >= items[2].getType().getMaxStackSize()))
				continue;
			
			if(recipe.minTime == 0.0) // instant smelting
			{
				furnace.setCookTime((short)199);
			}
			else if(recipe.minTime > 0.0)
			{
				time = entry.getValue();
				furnaceTime = furnace.getCookTime();
				
				if(time >= 199 || furnaceTime == 0 || furnaceTime > 199) // reset
				{
					Main.furnacesSmelting.put(entry.getKey(), 1.0);
				}
				else
				{
					time = time + ((10 / Main.cfgFurnaceTicks) / recipe.getTime());
					
					furnace.setCookTime((short)time);
					
					Main.furnacesSmelting.put(entry.getKey(), time);
				}
			}
		}
	}
}